var assert = require('assert');
global._ = require('../../../node_modules/underscore');
require('../loader')('lib/invalidator.js');

suite('Invalidator - .updateModifierToFields()', function() {
  test('update only operations', function() {
    var modifier = {$set: {aa: 10, bb: 20}};
    var fields = new Meteor.SmartInvalidator.updateModifierToFields(modifier);
    assert.deepEqual(fields, {
      remove: {},
      update: {aa: 1, bb: 1}
    });
  });

  test('remove only operations', function() {
    var modifier = {$unset: {aa: 10, bb: 20}};
    var fields = new Meteor.SmartInvalidator.updateModifierToFields(modifier);
    assert.deepEqual(fields, {
      update: {},
      remove: {aa: 1, bb: 1}
    });
  });

  test('multiple operations', function() {
    var modifier = {
      $set: {aa: 10, bb: 20},
      $inc: {cc: 20}
    };
    var fields = new Meteor.SmartInvalidator.updateModifierToFields(modifier);
    assert.deepEqual(fields, {
      remove: {},
      update: {aa: 1, bb: 1, cc: 1}
    });
  });

  test('update and remove operations', function() {
    var modifier = {$unset: {aa: 10, bb: 20}, $push: {bb: [20], ck: 30}, $set: {k: 20, ck: 29}};
    var fields = new Meteor.SmartInvalidator.updateModifierToFields(modifier);
    assert.deepEqual(fields, {
      update: {bb: 1, ck: 1, k: 1},
      remove: {aa: 1, bb: 1}
    });
  });

  test('$unset in nested object', function() {
    var modifier = {$unset: {'aa.uu': 10, bb: 20}};
    var fields = new Meteor.SmartInvalidator.updateModifierToFields(modifier);
    assert.deepEqual(fields, {
      update: {aa: 1},
      remove: {bb: 1}
    });
  });

  test('no modifiers - just direct update', function() {
    var modifier = {aa: 10, bb: 10};
    var fields = new Meteor.SmartInvalidator.updateModifierToFields(modifier);
    assert.deepEqual(fields, {
      update: {aa: 1, bb: 1},
      remove: {}
    });
  });

  test('empty $set', function() {
    var modifier = {$set: {}};
    var fields = new Meteor.SmartInvalidator.updateModifierToFields(modifier);
    assert.deepEqual(fields, {
      remove: {},
      update: {}
    });
  });

  test('$rename', function() {
    var modifier = {$rename: {'aa': 'bb'}};
    var fields = new Meteor.SmartInvalidator.updateModifierToFields(modifier);
    assert.deepEqual(fields, {
      update: {bb: 1},
      remove: {aa: 1}
    });
  });

  test('$rename nested object', function() {
    var modifier = {$rename: {'aa.vv': 'aa.cd'}};
    var fields = new Meteor.SmartInvalidator.updateModifierToFields(modifier);
    assert.deepEqual(fields, {
      update: {aa: 1},
      remove: {}
    });
  });

  test('$rename nested object with relocating', function() {
    var modifier = {$rename: {'aa.vv': 'kk.cd'}};
    var fields = new Meteor.SmartInvalidator.updateModifierToFields(modifier);
    assert.deepEqual(fields, {
      update: {aa: 1, kk: 1},
      remove: {}
    });
  });
});